// My_hit class DECLARATION and DEFINITION
// software by Gianluca Aglieri Rinella
// Doctoral Student at CERN
// European Organization for Nuclear Research
//
// with permission to use it freely but reproducing this comment
// including when reused, follow GPL and I will be happy
//
// last modified on 01/09/05
// C++ classes to store and work with HPD hits and hit cluster
// 
// class My_hit is a simple but useful hit description class
// it stores a hit by its row and col coordinates
// wrapped together
//
// to use it include this file from your source
// this software is using only very standard C++ classes and include files
// 
//
// Class My_hit
//
// FULL INTERFACE
//
// COMPULSORY constructor: My_hit (int row, int col);
//
// inline int GetRow();
// inline int GetCol();
// inline int SetRow(int row);
// inline int SetCol(int col);
// 
// bool IsAdiacentTo (My_hit &hit);
// bool IsTheSame (My_hit& hit);
// bool IsAdiacentOnSameRow (My_hit &hit);
// bool IsAdiacentOnSameColumn (My_hit &hit);
// bool operator ==  (My_hit& hit);  added on 1/09/2005, does the same
// thing of IsTheSame member function
//
// file also contains implementation of
// My_hit_tools::Eliminate_twins method,
// to eliminate eventual redundant (i.e. with same coordinates)
// hits in a vector of hits

#ifndef MY_HIT
#define MY_HIT

#include <stdlib.h>
#include <iostream>
#include <vector>

using namespace std;

// the minimum and maximum row and column allowed values, global constants
// change here in case of lhcb mode
const int minrow = 0;
const int maxrow = 255; // 31 for lhcb mode, 255 for alice mode.
const int mincol = 0;
const int maxcol = 31;

// Class My_hit, simply wraps row and col coordinates and provides a
// useful interface
class My_hit {

  // data members
  int row_num; 
  int col_num; 

public :
  
// NON default constructor follows: it is compulsory to call it and provide proper
//  coordinates check, complaining if out of range.
// No default constructor provided, so compulsory to use sintax:
// My_hit new_hit(RR, CC);
// to allocate an object on the stack or
// My_hit *new_hit_pointer = new My_hit(RR,CC);
// to allocate it on the heap and get a pointer

My_hit (int row, int col) {

    if (row>=minrow && row<=maxrow && col>=mincol && col<=maxcol) {
      row_num = row; col_num = col;
    }
    else {
      row_num = -1 ; col_num = -1; cout << "My_hit object initialized with unmeaningful coordinates, out of range: row=" << row << " col=" << col << endl;
    }

  }


  // getters, to read out coordinates
  inline int GetRow() { return row_num; }
  inline int GetCol() { return col_num; }
 
 // setters, return assigned value if OK, -1 if wrong
  inline int SetRow(int row) {
    if (row>=minrow && row<=maxrow) {row_num=row; return row_num;}
    else {return -1;} // row num out of range
  }
  inline int SetCol(int col) {
    if (col>=mincol && col<=maxcol) {col_num=col; return col_num;}
    else {return -1;} // row num out of range
  }

  // some member functions to implement adiacency checks
  //

  // two My_hits h1 and h2 can be adiacent on the same row, tell me if
  // true, example of call:
  // h1.IsAdiacentOnSameRow(h2);
  //

  bool IsAdiacentOnSameRow (My_hit &hit) {
     // if on same Row but on adiacent or same column...
    if (row_num==hit.GetRow() && 
	(col_num==hit.GetCol() ||
	 col_num==hit.GetCol()+1 ||
	 col_num==hit.GetCol()-1)) {return true;}
    else return false;  
}
  
  // or on the same column, tell me if it is true
  bool IsAdiacentOnSameColumn (My_hit &hit) {
    if (col_num==hit.GetCol() && 
	(row_num==hit.GetRow() ||
	 row_num==hit.GetRow()+1 ||
	 row_num==hit.GetRow()-1)) {return true;}
    else return false;  
}

  // or just adiacent, no matter how
  // NOTE: diagonally adiacent hits are not considered adiacent in
  // these
  // implementations! This will affect also the clustering function
  // using this member functions
  bool IsAdiacentTo (My_hit &hit) {
   
    // if on same Row but on adiacent or same column they are adiacent
    if (row_num==hit.GetRow() && 
	(col_num==hit.GetCol() ||
	 col_num==hit.GetCol()+1 ||
	 col_num==hit.GetCol()-1)) {return true;}
    
    // if on same col but on adiacent or same row they are adiacent
    if (col_num==hit.GetCol() && 
	(row_num==hit.GetRow() ||
	 row_num==hit.GetRow()+1 ||
	 row_num==hit.GetRow()-1)) {return true;}
    
    // none condition verified
    return false;
  }

  // I discovered that is useful to compare two hits for the
  // clustering
  // to know if in fact they represent the same hit, i.e. if they have
  // same coordinates.
  bool IsTheSame (My_hit& hit) {
    if (col_num==hit.GetCol() && row_num==hit.GetRow()) {
      return true;
    }
    return false;
  }

  // previous function can be also done by overloading
  // the == operator, 
  // but I never tested it
  // added on 1/09/2005

  bool operator ==  (My_hit& hit) {
    if (col_num==hit.GetCol() && row_num==hit.GetRow()) {
      return true;
    }
    return false;
  }

}; // end of class My_hit












// My_hit_tools::Eliminate_twins
// is an auxiliar method to filter out hits present twice into a hit
// list,
// i.e. it searches into a vector of My_hit if there are hits with
// same coordinates and eliminates redundant copies.
// It is not strictly a My_hit member method, it does not act on an object, but requires 
// a list as argument
// To use the method one has to call it with syntax:
// My_hit_tools::Eliminate_twins(hit_vector);
// or after a 
// using namespace My_hit_tools directive
//
namespace My_hit_tools {
 
void Eliminate_twins(vector<My_hit > &hit_list) {

  if (hit_list.size()>1) { // only if there are at least two hits in
                           // the list.. 
 
    vector<My_hit >::iterator hit_list_iter1, hit_list_iter2;

    for (hit_list_iter1=hit_list.begin();
	 hit_list_iter1<hit_list.end(); hit_list_iter1++) {

      for (hit_list_iter2=hit_list_iter1+1;
	   hit_list_iter2<hit_list.end(); hit_list_iter2++) {

    if( (*hit_list_iter1).IsTheSame((*hit_list_iter2)) == true) { // is the same 
      // kick element out from the list
      hit_list.erase(hit_list_iter2, hit_list_iter2+1);
    }

      }
    }
  }
}


 /*
 void Eliminate_twins(vector<My_hit > &hit_list) {

  if (hit_list.size()>1) { // only if there are at least two hits in
                           // the list.. 
  bool found = false;
  for (int i = hit_list.size()-1; i>0 ; i--) { //scan from last to first
    found = false; // not yet found a twin
  for (int j = i-1; j>=0 ; j--) { // scan from previous to first
    if(hit_list[i].IsTheSame(hit_list[j])==true) { // is the same 
	found = true;
      }
    }
    if (found==true) {
      hit_list.pop_back(); // ok, we got a twin...
     // remove last element, it had a twin in the list
    }
  }
  }

} // end of method Eliminate_twins
  */

}// end of namespace My_hit_tools

#endif

